#include "gethash.h"
#include "gethash_i.c"

class CGetHash : public IGetHash
{
        ULONG _nRefCnt;
    public:
        CGetHash()
        {
            _nRefCnt = 1;
        }
        
        HRESULT WINAPI GetHash(BSTR szStr, LONG* pOut)
        {
            DWORD digist = 0;
            while(*szStr)
            {
                digist = (digist<<25 | digist>>7);

                digist += *szStr;

                szStr++;
            }
            *pOut = (LONG)digist;
            return S_OK;
        }
        
        ULONG WINAPI AddRef()
        {
            return InterlockedIncrement(&_nRefCnt);
        }
        
        ULONG WINAPI Release()
        {
            int refCnt = InterlockedDecrement(&_nRefCnt);
            if (refCnt == 0) delete this;
            return refCnt;
        }
        
        HRESULT WINAPI QueryInterface(const GUID& iid, void** pOut)
        {
            if (iid == IID_IGetHash)
                *pOut = this;
            else if (iid == IID_IUnknown)
                *pOut = this;
            else {
                *pOut = 0;
                return E_NOINTERFACE;
            }
            AddRef();
            return S_OK;
        }
};

class CClassFactory : public IClassFactory
{
    public:
        ULONG WINAPI AddRef() { return 2; }
        
        ULONG WINAPI Release() { return 1; }
        
        HRESULT WINAPI QueryInterface(const GUID& iid, void** pOut)
        {
            if (iid == IID_IUnknown)
                *pOut = this;
            else if (iid == IID_IClassFactory)
                *pOut = this;
            else {
                *pOut = 0;
                return E_NOINTERFACE;
            }
            return S_OK;
        }
        
        HRESULT WINAPI CreateInstance(IUnknown* pOuter, const GUID& iid, void** pOut)
        {
            if (pOuter != NULL) {
                *pOut = 0;
                return CLASS_E_NOAGGREGATION;
            } else {
                if (iid == IID_IGetHash || iid == IID_IUnknown) {
                    CGetHash* gh = new CGetHash();
                    HRESULT r = gh->QueryInterface(iid, pOut);
                    gh->Release();
                    return r;
                } else {
                    *pOut = 0;
                    return E_NOINTERFACE;
                }
            }
        }
        
        HRESULT WINAPI LockServer(BOOL bLock) { return S_OK; }
};

static CClassFactory s_factory;

// {E541F90D-16E1-43D0-B115-23AAAE0573DB}
static const GUID CLSID_CGetHash = 
{ 0xe541f90d, 0x16e1, 0x43d0, { 0xb1, 0x15, 0x23, 0xaa, 0xae, 0x5, 0x73, 0xdb } };

#pragma comment(linker, "/export:DllGetClassObject=_DllGetClassObject@12")
extern "C"
HRESULT CALLBACK DllGetClassObject(const GUID& clsid, const GUID& iid, void** pOut)
{
    if (clsid == CLSID_CGetHash && iid == IID_IClassFactory) {
        *pOut = &s_factory;
        return S_OK;
    } else {
        pOut = 0;
        return CLASS_E_CLASSNOTAVAILABLE;
    }
}

#pragma comment(linker, "/export:DllRegisterServer=_DllRegisterServer@0")
extern "C" HRESULT CALLBACK DllRegisterServer()
{
}

#pragma comment(linker, "/export:_DllUnregisterServer=_DllUnregisterServer@0")
extern "C" HRESULT CALLBACK DllUnregisterServer()
{
}
















